@@ -195,7 +195,7 @@ static void GE_send_key(GaimAccount *acct, const char *name, int asError, gchar
       return;
    }
 
-   serv_send_im(acct->gc, name, msg, 0);
+   serv_send_im(acct->gc, name, msg, GAIM_CONV_IM_AUTO_RESP);
    g_string_free(key_str, TRUE);
 }
 
@@ -226,7 +226,7 @@ static crypt_key *GE_get_key(GaimConnection *gc, const char *name) {
       sprintf(tmpmsg, "%s%s%s", header, ": Send Key", footer);
 
       gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "Sending: %s\n", tmpmsg);
-      serv_send_im(gc, name, tmpmsg, 0);
+      serv_send_im(gc, name, tmpmsg, GAIM_CONV_IM_AUTO_RESP);
       return 0;
    }
 
@@ -387,6 +387,7 @@ void GE_show_stored_msgs(GaimAccount*acct, const char* who, char** return_msg) {
                                      GAIM_MESSAGE_RECV, time((time_t)NULL));
                   gaim_conv_window_flash(gaim_conversation_get_window(conv));
                   g_free(*return_msg);
+                  *return_msg = 0;
                } else {
                   *return_msg = tmp_msg;
                }
@@ -1030,10 +1031,19 @@ static void GE_headers_init() {
    g_hash_table_insert(notify_table, g_strdup("prpl-oscar"),
                        g_strdup("<A HREF=\"Gaim-Encryption Capable\"></A>"));
 
+/* If jabber stops stripping HTML, we can go back to these headers */
+/*    g_hash_table_insert(header_table, g_strdup("prpl-jabber"), */
+/*                        g_strdup("*** Encrypted with the Gaim-Encryption plugin <A HREF='")); */
+/*    g_hash_table_insert(footer_table, g_strdup("prpl-jabber"), */
+/*                        g_strdup("'></A>")); */
+/*    g_hash_table_insert(notify_table, g_strdup("prpl-jabber"), */
+/*                        g_strdup("<A HREF='Gaim-Encryption Capable'> </A>")); */
+
+
    g_hash_table_insert(header_table, g_strdup("prpl-jabber"),
-                       g_strdup("*** Encrypted with the Gaim-Encryption plugin <A HREF='"));
+                       g_strdup("*** Encrypted with the Gaim-Encryption plugin "));
    g_hash_table_insert(footer_table, g_strdup("prpl-jabber"),
-                       g_strdup("'></A>"));
+                       g_strdup(" "));
    g_hash_table_insert(notify_table, g_strdup("prpl-jabber"),
                        g_strdup("<A HREF='Gaim-Encryption Capable'> </A>"));
 
@@ -1069,7 +1079,7 @@ static void init_prefs() {
    gaim_prefs_add_none("/plugins/gtk");
    gaim_prefs_add_none("/plugins/gtk/encrypt");
    
-   gaim_prefs_add_bool("/plugins/gtk/encrypt/accept_unknown_key", TRUE);
+   gaim_prefs_add_bool("/plugins/gtk/encrypt/accept_unknown_key", FALSE);
    gaim_prefs_add_bool("/plugins/gtk/encrypt/accept_conflicting_key", FALSE);
    gaim_prefs_add_bool("/plugins/gtk/encrypt/encrypt_response", TRUE);
    gaim_prefs_add_bool("/plugins/gtk/encrypt/broadcast_notify", FALSE);
@@ -1091,11 +1101,10 @@ static gboolean GE_plugin_load(GaimPlugin *h) {
 
    if (strcmp(gaim_core_get_version(), VERSION) != 0) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 _("Compiled with '%s', running with '%s'."
-                   "  Gaim-Encryption will probably not run right.\n"),
+                 "Compiled with Gaim v'%s', running with v'%s'.\n",
                  VERSION, gaim_core_get_version());
-      GE_error_window(_("Gaim-Encryption plugin was compiled with a different "
-                        "version of Gaim.  You may experience problems."));
+      /* GE_error_window(_("Gaim-Encryption plugin was compiled with a different " */
+      /*                   "version of Gaim.  You may experience problems.")); */
    }
 
    init_prefs();
