@@ -17,6 +17,22 @@
 
 /* #define GAIM_PLUGINS */
 
+#include <glib.h>
+#if GLIB_CHECK_VERSION(2,6,0)
+#	include <glib/gstdio.h>
+#else
+#	define g_freopen freopen
+#	define g_fopen fopen
+#	define g_rmdir rmdir
+#	define g_remove remove
+#	define g_unlink unlink
+#	define g_lstat lstat
+#	define g_stat stat
+#	define g_mkdir mkdir
+#	define g_rename rename
+#	define g_open open
+#endif
+
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkplug.h>
@@ -277,14 +293,14 @@ static KeyCheckVal GE_check_known_key(const char* filename, key_ring_data* key)
    int fd;
    int found_name = 0;
    
-   g_snprintf(path, sizeof(path), "%s%s%s", gaim_home_dir(), G_DIR_SEPARATOR_S, filename);
+   g_snprintf(path, sizeof(path), "%s%s%s", gaim_user_dir(), G_DIR_SEPARATOR_S, filename);
       
    gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "Checking key file %s for name %s\n", path, key->name);
 
    /* check file permissions */
    if (stat(path, &fs) == -1) {
       /* file doesn't exist, so make it */
-      fd = open(path, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+      fd = g_open(path, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
       if (fd == -1) {
          /* Ok, maybe something else strange is going on... */
          gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error trying to create a known key file\n");
@@ -315,7 +331,7 @@ static KeyCheckVal GE_check_known_key(const char* filename, key_ring_data* key)
    /*   gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "built line '%s'\n", line_str->str); */
 
    /* look for key in file */   
-   if( (fp = fopen(path, "r")) != NULL ) {
+   if( (fp = g_fopen(path, "r")) != NULL ) {
       while (!feof(fp)) {
          fgets(line, sizeof(line), fp);
          /* gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "checking line '%s'\n", line); */
@@ -359,11 +375,13 @@ void GE_add_key_to_file(const char *filename, key_ring_data* key) {
    struct stat fdstat;
    
    gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "Saving key to file:%s:%p\n", key->name, key->account);
-   g_snprintf(path, sizeof(path), "%s%s%s", gaim_home_dir(), G_DIR_SEPARATOR_S, filename);
+   g_snprintf(path, sizeof(path), "%s%s%s", gaim_user_dir(), G_DIR_SEPARATOR_S, filename);
 
-   fd = open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+   fd = g_open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
    if (fd == -1) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error opening key file %s for write\n", path);
+      /* WIN32 doesn't have user-based file permissions, so skips this */
+#ifdef S_IRWXG
       if (chmod(path, S_IRUSR | S_IWUSR) == -1) {
 
          gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Unable to change file mode, aborting\n");
@@ -372,7 +390,8 @@ void GE_add_key_to_file(const char *filename, key_ring_data* key) {
          GE_ui_error(errbuf);
          return;
       }
-      fd = open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+#endif
+      fd = g_open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
       if (fd == -1) {
          gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Changed mode, but still wonky.  Aborting.\n");
          g_snprintf(errbuf, sizeof(errbuf),
@@ -415,7 +434,7 @@ void GE_add_key_to_file(const char *filename, key_ring_data* key) {
    if (feof(fp)) c = '\n';  /*if file is empty, we don't need to write a \n */
    fclose(fp);
    
-   fd = open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+   fd = g_open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
    fp = fdopen(fd, "a+");
    fseek(fp, 0, SEEK_END);   /* should be unnecessary, but needed for WIN32 */
    
@@ -449,10 +468,10 @@ void GE_del_one_key_from_file(const char *filename, int key_num, const char *nam
    GE_escape_name(normalized_start);
    g_string_append_printf(normalized_start, " ");
 
-   g_snprintf(path, sizeof(path), "%s%s%s", gaim_home_dir(), G_DIR_SEPARATOR_S, filename);
+   g_snprintf(path, sizeof(path), "%s%s%s", gaim_user_dir(), G_DIR_SEPARATOR_S, filename);
    
    /* Look for name in the file.  If it's not there, we're done */
-   fp = fopen(path, "r");
+   fp = g_fopen(path, "r");
 
    if (fp == NULL) {
       g_string_free(line_start, TRUE);
@@ -487,7 +506,7 @@ void GE_del_one_key_from_file(const char *filename, int key_num, const char *nam
    g_snprintf(tmp_path, sizeof(tmp_path), "%s.tmp", path);
    rename(path, tmp_path);
    
-   fd = open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+   fd = g_open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
    if (fd == -1) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error opening key file %s\n", path);
       perror("Error opening key file");
@@ -499,7 +518,7 @@ void GE_del_one_key_from_file(const char *filename, int key_num, const char *nam
    }
    fp = fdopen(fd, "a+");
    
-   tmp_fp = fopen(tmp_path, "r");
+   tmp_fp = g_fopen(tmp_path, "r");
    if (tmp_fp == NULL) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Wah!  I moved a file and now it is gone\n");
       fclose(fp);
@@ -551,10 +570,10 @@ void GE_del_key_from_file(const char *filename, const char *name, GaimAccount *a
    GE_escape_name(normalized_start);
    g_string_append_printf(normalized_start, " ");
 
-   g_snprintf(path, sizeof(path), "%s%s%s", gaim_home_dir(), G_DIR_SEPARATOR_S, filename);
+   g_snprintf(path, sizeof(path), "%s%s%s", gaim_user_dir(), G_DIR_SEPARATOR_S, filename);
    
    /* Look for name in the file.  If it's not there, we're done */
-   fp = fopen(path, "r");
+   fp = g_fopen(path, "r");
 
    if (fp == NULL) {
       g_string_free(line_start, TRUE);
@@ -583,7 +602,7 @@ void GE_del_key_from_file(const char *filename, const char *name, GaimAccount *a
    g_snprintf(tmp_path, sizeof(tmp_path), "%s.tmp", path);
    rename(path, tmp_path);
    
-   fd = open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+   fd = g_open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
    if (fd == -1) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error opening key file %s\n", path);
       perror("Error opening key file");
@@ -595,7 +614,7 @@ void GE_del_key_from_file(const char *filename, const char *name, GaimAccount *a
    }
    fp = fdopen(fd, "a+");
    
-   tmp_fp = fopen(tmp_path, "r");
+   tmp_fp = g_fopen(tmp_path, "r");
    if (tmp_fp == NULL) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Wah!  I moved a file and now it is gone\n");
       fclose(fp);
@@ -633,8 +652,8 @@ key_ring * GE_load_keys(const char *filename) {
    GaimAccount* account;
 
 
-   g_snprintf(path, sizeof(path), "%s%s%s", gaim_home_dir(), G_DIR_SEPARATOR_S, filename);
-   if( (fp = fopen(path, "r")) != NULL ) {
+   g_snprintf(path, sizeof(path), "%s%s%s", gaim_user_dir(), G_DIR_SEPARATOR_S, filename);
+   if( (fp = g_fopen(path, "r")) != NULL ) {
       do {
       	 /* 7999 = MAX_KEY_STORLEN - 1 */
          rv = fscanf(fp, "%163s %9s %9s %7999s\n", nameacct, proto_name,
@@ -654,7 +673,9 @@ key_ring * GE_load_keys(const char *filename) {
             account = gaim_accounts_find(name, nameaccount_split[1]);
 
             gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "load_keys: name(%s), protocol (%s): %p\n",
-                       nameaccount_split[0], nameaccount_split[1], account);
+                       nameaccount_split[0],
+                       ((nameaccount_split[1]) ? nameaccount_split[1] : "none"),
+                        account);
 
             gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "%p\n", gaim_account_get_protocol_id(account));
 
