@@ -19,6 +19,22 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <glib.h>
+#if GLIB_CHECK_VERSION(2,6,0)
+#	include <glib/gstdio.h>
+#else
+#	define g_freopen freopen
+#	define g_fopen fopen
+#	define g_rmdir rmdir
+#	define g_remove remove
+#	define g_unlink unlink
+#	define g_lstat lstat
+#	define g_stat stat
+#	define g_mkdir mkdir
+#	define g_rename rename
+#	define g_open open
+#endif
+
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkplug.h>
@@ -54,7 +70,7 @@ void GE_convert_legacy_prefs() {
    
    char* filename = g_build_filename(gaim_user_dir(), key_file, NULL);
 
-   FILE* fp = fopen(filename, "r");
+   FILE* fp = g_fopen(filename, "r");
 
    gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "Checking for old prefs file (%s)...\n", filename);
 
