@@ -19,10 +19,9 @@
 #ifndef PREFS_H
 #define PREFS_H
 
-#include "src/debug.h"
-#include "src/internal.h"
-#include "src/ui.h"
-#include "src/gaim.h"
+#include <debug.h>
+#include <gtkdialogs.h>
+#include <gaim.h>
 
 void GE_convert_legacy_prefs(void);
 
