@@ -19,22 +19,12 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkplug.h>
 
-#include <nspr.h>
-#include <nss.h>
-#include <pk11func.h>
-#include <keyhi.h>
-#include <nssb64.h>
-
-#include "rsa_nss.h"
-
-#include "src/debug.h"
-#include "src/internal.h"
-#include "src/gaim.h"
-#include "src/ui.h"
+#include <debug.h>
+#include <gaim.h>
+#include <gtkdialogs.h>
 
 #include "glib/gmain.h"
 
-
 #include <string.h>
 #include <assert.h>
 
@@ -42,6 +32,18 @@
 #include "win32dep.h"
 #endif
 
+#include "rsa_nss.h"
+
+#include <nspr.h>
+#include <nss.h>
+#include <ssl.h>
+#include <secmod.h>
+#include <pk11func.h>
+#include <keyhi.h>
+#include <nssb64.h>
+
+
+#include "nls.h"
 #include "nss_mgf1.h"
 #include "nss_oaep.h"
 #include "nss_pss.h"
@@ -79,18 +81,37 @@ void rsa_nss_test(crypt_key *pub, crypt_key *priv);
 
 gboolean rsa_nss_init() {
 
+   gboolean nss_loaded_ok = FALSE;
+
   	GaimPlugin *plugin = gaim_plugins_find_with_name("NSS");
 
-   if (plugin == NULL) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Can't load the NSS plugin\n");
-
-      GE_error_window("Gaim was not compiled with the NSS plugin enabled.  "
-                      "Gaim-Encryption requires the NSS plugin to function.");
-      return FALSE;
+   if (plugin != NULL) {
+      nss_loaded_ok = gaim_plugin_is_loaded(plugin);
+      if (!nss_loaded_ok) {
+         nss_loaded_ok = gaim_plugin_load(plugin);
+      }
    }
 
-	if (!gaim_plugin_is_loaded(plugin)) {
-		gaim_plugin_load(plugin);
+   if (!nss_loaded_ok) {
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Initializing NSS without Gaim support\n");
+      /* Gaim doesn't seem to have NSS support: try to load it ourselves: */
+      PR_Init(PR_SYSTEM_THREAD, PR_PRIORITY_NORMAL, 1);
+      NSS_NoDB_Init(NULL);
+      
+      /* TODO: Fix this so autoconf does the work trying to find this lib. */
+      SECMOD_AddNewModule("Builtins",
+#ifndef _WIN32
+                          LIBDIR "/libnssckbi.so",
+#else
+                          "nssckbi.dll",
+#endif
+                          0, 0);
+      NSS_SetDomesticPolicy();
+
+      /*       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", _("Can't load the NSS plugin\n")); */
+      /*       GE_error_window(_("Gaim was not compiled with the NSS plugin enabled.  " */
+      /*                       "Gaim-Encryption requires the NSS plugin to function.")); */
+      /*       return FALSE; */
    }
 
    rsa_nss_proto = g_malloc(sizeof(crypt_proto));
@@ -202,7 +223,7 @@ void rsa_nss_gen_key_pair(crypt_key **pub_key, crypt_key **priv_key,
                           const char* name, int keysize) {
 
    GtkWidget *status_window, *main_box, *label1, *label2;
-   char labelText[BUF_LONG];
+   char labelText[1000];
    PK11RSAGenParams rsaParams;
    PK11SlotInfo *slot;
 
@@ -220,10 +241,10 @@ void rsa_nss_gen_key_pair(crypt_key **pub_key, crypt_key **priv_key,
    gtk_container_add(GTK_CONTAINER(status_window), main_box);
    gtk_widget_show(main_box);
    
-   g_snprintf(labelText, sizeof(labelText), "Generating RSA Key Pair for %s",
+   g_snprintf(labelText, sizeof(labelText), _("Generating RSA Key Pair for %s"),
               name);
    label1 = gtk_label_new (labelText);
-   label2 = gtk_label_new ("This may take a little bit...");
+   label2 = gtk_label_new (_("This may take a little bit..."));
    
    gtk_container_add (GTK_CONTAINER (main_box), label1);
    gtk_widget_show(label1);
@@ -258,7 +279,7 @@ void rsa_nss_gen_key_pair(crypt_key **pub_key, crypt_key **priv_key,
 
    if (!(*priv_key)->store.rsa_nss.priv) {
       gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Could not generate key.  NSS Error: %d\n",
+                 _("Could not generate key.  NSS Error: %d\n"),
                  PORT_GetError());
       exit(0);
    }
@@ -414,6 +435,8 @@ crypt_key* rsa_nss_parse_sent_key(unsigned char *key_str) {
    
    key = rsa_nss_make_key_from_str(split_key[1]);
 
+   if (key == 0) return 0;
+
    GE_str_to_nonce(&key->store.rsa_nss.nonce, split_key[0]);
    gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "Received Nonce with key: %s\n", split_key[0]);
 
@@ -469,7 +492,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
    // Check for public key part, and get it:
 
    if (split_key[cur_piece] == 0) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error parsing RSANSS key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 1, _("Error parsing RSANSS key\n"));
       g_free(key);
       g_strfreev(split_key);
       return 0;
@@ -478,8 +502,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
    wrappedKey = NSSBase64_DecodeBuffer(0, 0, split_key[cur_piece], strlen(split_key[cur_piece]));
 
    if (wrappedKey == 0) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Error decoding Base64 RSANSS Public key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 2, _("Error parsing RSANSS key\n"));
       g_free(key);
       g_strfreev(split_key);
       return 0;
@@ -491,8 +515,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
    PORT_Free(wrappedKey);
    
    if (certPubKeyInfo == NULL) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Error Base64 decoding RSANSS Public key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 3, _("Error parsing RSANSS key\n"));
       g_free(key);
       g_strfreev(split_key);
       return 0;
@@ -501,8 +525,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
    key->store.rsa_nss.pub = SECKEY_ExtractPublicKey(certPubKeyInfo);
 
    if (key->store.rsa_nss.pub == NULL) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Error decoding RSANSS Public key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 4, _("Error parsing RSANSS key\n"));
       g_free(key);
       g_strfreev(split_key);
       return 0;
@@ -531,8 +555,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
 
    if (split_key[++cur_piece] == 0) {
       /* only part of a private key */
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Error parsing RSANSS Private key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 5, _("Error parsing RSANSS key\n"));
       g_free(key);
       g_strfreev(split_key);
       return 0;
@@ -542,8 +566,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
                                         strlen(split_key[cur_piece]));
 
    if ((iv == 0) || (wrappedKey == 0)) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Error parsing Base64 in RSANSS Private Key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 6, _("Error parsing RSANSS key\n"));
       g_free(key);
       g_strfreev(split_key);
       return 0;
@@ -561,8 +585,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
                                PK11_OriginUnwrap, CKA_WRAP, &symKeyItem, NULL);
 
    if (!symKey) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
-                 "Error creating symmetric key: unable to load private key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 7, _("Error parsing RSANSS key\n"));
       g_strfreev(split_key);
 
       SECKEY_DestroyPublicKey(key->store.rsa_nss.pub);
@@ -581,8 +605,8 @@ crypt_key* rsa_nss_make_key_from_str(unsigned char *key_str){
    SECITEM_FreeItem (pubKeyValue, PR_TRUE);
 
    if (key->store.rsa_nss.priv == 0) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", 
-                 "Error decoding RSANSS Private key\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "(%d) %s",
+                 8, _("Error parsing RSANSS key\n"));
       g_strfreev(split_key);
 
       SECKEY_DestroyPublicKey(key->store.rsa_nss.pub);
@@ -772,7 +796,7 @@ int rsa_nss_sign(unsigned char** signed_msg, unsigned char* msg, int msg_len,
                            tmp_sig, modulus_len);
 
    if (rv != SECSuccess) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Failed PubDecrypt\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "PK11_PubDecryptRaw Failed\n");
       g_free(*signed_msg);
       *signed_msg = 0;
       return 0;
@@ -816,7 +840,7 @@ int rsa_nss_auth(unsigned char** authed, unsigned char* msg, int msg_len,
                            modulus_len, 0);
 
    if (rv != SECSuccess) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Failed PubEncrypt\n");
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "PK11_PubEncryptRaw Failed\n");
       g_free(tmp_sig);
       return -1;
    }
@@ -828,7 +852,7 @@ int rsa_nss_auth(unsigned char** authed, unsigned char* msg, int msg_len,
    g_free(tmp_sig);
    
    if (!verified) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Bad signature on message (len %d, mod %d)\n",
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", _("Bad signature on message (len %d, mod %d)\n"),
                  msg_len, modulus_len);
       return -1;
    }
